/*
 * Designed and developed in 2022-2024 by William Walker (wnwalker@ucsc.edu)
 * Copyright 2022-2024 The Regents of the University of California. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package edu.ucsc.its.temerity.shared.ui

import androidx.compose.animation.AnimatedVisibility
import androidx.compose.animation.fadeIn
import androidx.compose.animation.fadeOut
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.text.KeyboardActions
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.outlined.Abc
import androidx.compose.material.icons.outlined.Cancel
import androidx.compose.material.icons.outlined.Email
import androidx.compose.material.icons.outlined.Person
import androidx.compose.material.icons.outlined.Save
import androidx.compose.material3.Checkbox
import androidx.compose.material3.FloatingActionButton
import androidx.compose.material3.Icon
import androidx.compose.material3.IconButton
import androidx.compose.material3.OutlinedTextField
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.focus.FocusDirection
import androidx.compose.ui.platform.LocalFocusManager
import androidx.compose.ui.text.input.ImeAction
import androidx.compose.ui.text.input.KeyboardCapitalization
import androidx.compose.ui.text.input.KeyboardType
import androidx.compose.ui.unit.dp
import androidx.lifecycle.viewModelScope
import com.composables.composetheme.ComposeTheme
import com.composables.composetheme.material3.colorScheme
import com.composables.composetheme.material3.typography
import edu.ucsc.its.temerity.remote.User
import edu.ucsc.its.temerity.shared.ui.elements.DropdownMenu
import edu.ucsc.its.temerity.shared.ui.elements.ScrollableContent
import edu.ucsc.its.temerity.shared.viewmodel.UserEditViewModel
import kotlinx.coroutines.launch
import org.jetbrains.compose.resources.stringResource
import org.koin.compose.viewmodel.koinViewModel
import org.koin.core.annotation.KoinExperimentalAPI
import temerity.shared.compose.generated.resources.Res
import temerity.shared.compose.generated.resources.save
import temerity.shared.compose.generated.resources.user_edit

@OptIn(KoinExperimentalAPI::class)
@Composable
fun UserEditScreen(user: User? = null) {
  val focusManager = LocalFocusManager.current
  val viewmodel: UserEditViewModel = koinViewModel()
  val viewModelScope = viewmodel.viewModelScope
  LaunchedEffect(user) {
    viewModelScope.launch { viewmodel.loadUser(user) }
  }
  val userEditState by viewmodel.models.collectAsState()

  Scaffold(
    floatingActionButton = {
      FloatingActionButton(onClick = { viewModelScope.launch { viewmodel.saveUserEdits() } }) {
        Icon(Icons.Outlined.Save, contentDescription = stringResource(Res.string.save))
      }
    },
  ) {
    ScrollableContent {
      item {
        Text(
          text = stringResource(Res.string.user_edit),
          modifier = Modifier.padding(start = 16.dp, end = 16.dp, top = 24.dp, bottom = 8.dp),
          color = ComposeTheme.colorScheme.onSurface,
          style = ComposeTheme.typography.titleMedium,
        )
      }
      item {
        val fieldName = "firstName"
        Row(modifier = Modifier.fillMaxWidth().padding(horizontal = 16.dp), verticalAlignment = Alignment.CenterVertically, horizontalArrangement = Arrangement.spacedBy(24.dp)) {
          Icon(Icons.Outlined.Person, null)
          OutlinedTextField(
            modifier = Modifier.weight(1f),
            label = { Text("First name") },
            value = userEditState.firstName,
            onValueChange = { viewModelScope.launch { viewmodel.updateField(fieldName, it) } },
            singleLine = true,
            trailingIcon = {
              AnimatedVisibility(visible = userEditState.firstName.isNotBlank(), enter = fadeIn(), exit = fadeOut()) {
                IconButton(onClick = { viewModelScope.launch { viewmodel.clearField(fieldName) } }) {
                  Icon(Icons.Outlined.Cancel, "Clear")
                }
              }
            },
            keyboardOptions = KeyboardOptions(keyboardType = KeyboardType.Text, imeAction = ImeAction.Next, capitalization = KeyboardCapitalization.Words),
            keyboardActions = KeyboardActions {
              focusManager.moveFocus(FocusDirection.Next)
            },
          )
        }
      }
      item {
        val fieldName = "lastName"
        Row(modifier = Modifier.fillMaxWidth().padding(horizontal = 16.dp), verticalAlignment = Alignment.CenterVertically, horizontalArrangement = Arrangement.spacedBy(24.dp)) {
          Box(Modifier.size(24.dp))
          OutlinedTextField(
            modifier = Modifier.weight(1f),
            label = { Text("Last name") },
            value = userEditState.lastName,
            onValueChange = { viewModelScope.launch { viewmodel.updateField(fieldName, it) } },
            singleLine = true,
            trailingIcon = {
              AnimatedVisibility(visible = userEditState.lastName.isNotBlank(), enter = fadeIn(), exit = fadeOut()) {
                IconButton(onClick = { viewModelScope.launch { viewmodel.clearField(fieldName) } }) {
                  Icon(Icons.Outlined.Cancel, "Clear")
                }
              }
            },
            keyboardOptions = KeyboardOptions(keyboardType = KeyboardType.Text, imeAction = ImeAction.Next, capitalization = KeyboardCapitalization.Words),
            keyboardActions = KeyboardActions {
              focusManager.moveFocus(FocusDirection.Next)
            },
          )
        }
      }
      item { Spacer(Modifier.height(4.dp)) }
      item {
        val fieldName = "loginId"
        Row(modifier = Modifier.fillMaxWidth().padding(horizontal = 16.dp), verticalAlignment = Alignment.CenterVertically, horizontalArrangement = Arrangement.spacedBy(24.dp)) {
          Icon(Icons.Outlined.Abc, null)
          OutlinedTextField(
            modifier = Modifier.weight(1f),
            label = { Text("Login ID") },
            value = userEditState.loginId,
            onValueChange = { viewModelScope.launch { viewmodel.updateField(fieldName, it) } },
            trailingIcon = {
              AnimatedVisibility(visible = userEditState.loginId.isNotBlank(), enter = fadeIn(), exit = fadeOut()) {
                IconButton(onClick = { viewModelScope.launch { viewmodel.clearField(fieldName) } }) {
                  Icon(Icons.Outlined.Cancel, "Clear")
                }
              }
            },
            keyboardOptions = KeyboardOptions(keyboardType = KeyboardType.Text, imeAction = ImeAction.Next),
            keyboardActions = KeyboardActions {
              focusManager.moveFocus(FocusDirection.Next)
            },
            singleLine = true,
          )
        }
      }
      item { Spacer(Modifier.height(4.dp)) }
      item {
        val fieldName = "emailAddress"
        Row(modifier = Modifier.fillMaxWidth().padding(horizontal = 16.dp), verticalAlignment = Alignment.CenterVertically, horizontalArrangement = Arrangement.spacedBy(24.dp)) {
          Icon(Icons.Outlined.Email, null)
          OutlinedTextField(
            modifier = Modifier.weight(1f),
            label = { Text("Email Address") },
            value = userEditState.emailAddress,
            onValueChange = { viewModelScope.launch { viewmodel.updateField(fieldName, it) } },
            trailingIcon = {
              AnimatedVisibility(visible = userEditState.emailAddress.isNotBlank(), enter = fadeIn(), exit = fadeOut()) {
                IconButton(onClick = { viewModelScope.launch { viewmodel.clearField(fieldName) } }) {
                  Icon(Icons.Outlined.Cancel, "Clear")
                }
              }
            },
            keyboardOptions = KeyboardOptions(keyboardType = KeyboardType.Email, imeAction = ImeAction.Done),
            keyboardActions = KeyboardActions {
              focusManager.clearFocus()
            },
            singleLine = true,
          )
        }
      }
      item { Spacer(Modifier.height(4.dp)) }
      item {
        Text(
          text = "Timezone",
          modifier = Modifier.padding(start = 16.dp, end = 16.dp, top = 24.dp, bottom = 8.dp),
          color = ComposeTheme.colorScheme.onSurface,
          style = ComposeTheme.typography.titleMedium,
        )
      }
      item {
        DropdownMenu(
          items = listOf(
            "America/Los_Angeles",
            "America/New_York",
            "Canada/East",
          ),
          onItemSelected = {},
          modifier = Modifier.height(40.dp).fillMaxWidth(),
        )
      }
      item {
        Text(text = "User Role", style = ComposeTheme.typography.titleMedium, modifier = Modifier.padding(start = 16.dp, end = 16.dp, top = 24.dp, bottom = 8.dp))
      }
      item {
        DropdownMenu(
          items = listOf(
            "Student",
            "Instructor",
            "IT Manager",
          ),
          onItemSelected = {},
          modifier = Modifier.height(40.dp).fillMaxWidth(),
        )
      }
      item {
        Text(
          text = "Extra Options",
          modifier = Modifier.padding(start = 16.dp, end = 16.dp, top = 24.dp, bottom = 8.dp),
          color = ComposeTheme.colorScheme.onSurface,
          style = ComposeTheme.typography.titleMedium,
        )
      }
      item {
        var selected by remember { mutableStateOf(false) }
        Row(verticalAlignment = Alignment.CenterVertically, horizontalArrangement = Arrangement.spacedBy(24.dp), modifier = Modifier.clickable { selected = selected.not() }.fillMaxWidth().padding(16.dp)) {
          Checkbox(checked = selected, onCheckedChange = null)
          Text(text = "Lock User")
        }
      }
    }
  }
}
