/*
 * Designed and developed in 2022-2024 by William Walker (wnwalker@ucsc.edu)
 * Copyright 2022-2024 The Regents of the University of California. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package edu.ucsc.its.temerity.shared.ui.elements

import androidx.compose.desktop.ui.tooling.preview.Preview
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.material.Text
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.Edit
import androidx.compose.material3.Icon
import androidx.compose.material3.IconButton
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.unit.dp
import com.composables.composetheme.ComposeTheme
import com.composables.composetheme.material3.colorScheme
import com.composables.composetheme.material3.typography
import edu.ucsc.its.temerity.remote.User
import org.jetbrains.compose.resources.stringResource
import temerity.shared.compose.generated.resources.Res
import temerity.shared.compose.generated.resources.user_edit

@Preview
@Composable
fun UserListPreview() {
  val mockUserList = listOf(
    User(),
    User(),
    User(),
  )
  PreviewThemeWrapper {
    UserList(userList = mockUserList) {}
  }
}

@Composable
fun UserList(userList: List<User>, onEditClick: (User) -> Unit) {
  ScrollableList(userList) {
    UserCardRow(user = it, onEditClick = { onEditClick(it) })
  }
}

@Composable
fun UserCardRow(user: User, onEditClick: () -> Unit) {
  CardRow(
    title = user.loginId,
    subtitle = "Role: ${user.userType}",
  ) {
    LazyColumn(
      verticalArrangement = Arrangement.spacedBy(16.dp),
      modifier = Modifier.height(190.dp),
      contentPadding = PaddingValues(vertical = 8.dp),
    ) {
      item {
        Row(
          verticalAlignment = Alignment.CenterVertically,
          horizontalArrangement = Arrangement.SpaceBetween,
          modifier = Modifier.fillMaxWidth(),
        ) {
          Column {
            Text(
              text = "Name: ${user.firstName} ${user.lastName}",
              color = ComposeTheme.colorScheme.primary,
              style = ComposeTheme.typography.bodyLarge,
            )
            Spacer(modifier = Modifier.width(4.dp).background(ComposeTheme.colorScheme.background))
            Text(
              text = "Login email: ${user.emailAddress}",
              color = ComposeTheme.colorScheme.primary,
              style = ComposeTheme.typography.bodyLarge,
            )
          }
          IconButton(
            onClick = onEditClick,
            modifier = Modifier.padding(8.dp),
          ) {
            Icon(imageVector = Icons.Default.Edit, contentDescription = stringResource(Res.string.user_edit))
          }
        }
      }
    }
  }
}
