/*
 * Designed and developed in 2022-2024 by William Walker (wnwalker@ucsc.edu)
 * Copyright 2022-2024 The Regents of the University of California. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package edu.ucsc.its.temerity.shared.viewmodel

import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.mutableStateOf
import androidx.lifecycle.viewModelScope
import edu.ucsc.its.temerity.shared.data.repository.PlatformRepository
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.launch
import org.koin.core.component.KoinComponent
import org.koin.core.component.inject

sealed interface LoginEvent {
  data object AttemptLogin : LoginEvent
  data object LoginSuccess : LoginEvent
  data class LoginFailure(val message: String) : LoginEvent
}

data class LoginState(
  val serviceToken: String,
)

class LoginViewModel : MoleculeViewModel<LoginEvent, LoginState>(), KoinComponent {

  private val repository: PlatformRepository by inject()
  private val serviceToken = mutableStateOf("")

  @Composable
  override fun models(events: Flow<LoginEvent>): LoginState {
    return loginScreenPresenter(events)
  }

  @Composable
  fun loginScreenPresenter(eventCollector: Flow<LoginEvent>): LoginState {
    LaunchedEffect(Unit) {
      eventCollector.collect { event ->
        when (event) {
          is LoginEvent.AttemptLogin -> {
            viewModelScope.launch(Dispatchers.IO) {
              try {
                // TODO: Update token and refresh platform client injected into repository here.
                events.tryEmit(LoginEvent.LoginSuccess)
              } catch (e: Exception) {
                events.tryEmit(LoginEvent.LoginFailure(e.message ?: "Unknown error"))
              }
            }
          }
          is LoginEvent.LoginSuccess -> {
          }
          is LoginEvent.LoginFailure -> {
          }
        }
      }
    }

    return LoginState("")
  }

  suspend fun tryLogin() {
    events.tryEmit(LoginEvent.AttemptLogin)
  }
}
