/*
 *     Designed and developed in 2022-2024 by William Walker (wnwalker@ucsc.edu)
 *     Copyright 2022-2024 The Regents of the University of California. All rights reserved.
 *
 *     This library is free software; you can redistribute it and/or
 *     modify it under the terms of the GNU Lesser General Public
 *     License as published by the Free Software Foundation; version 2.1 of the License.
 *
 *     This library is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *     Lesser General Public License for more details.
 *
 *     You should have received a copy of the GNU Lesser General Public
 *     License along with this library; if not, write to the Free Software
 *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
package edu.ucsc.its.temerity.di

import co.touchlab.kermit.NoTagFormatter
import co.touchlab.kermit.Severity
import co.touchlab.kermit.loggerConfigInit
import co.touchlab.kermit.platformLogWriter
import com.skydoves.sandwich.ktorfit.ApiResponseConverterFactory
import de.jensklingenberg.ktorfit.ktorfit
import edu.ucsc.its.temerity.PlatformClient
import edu.ucsc.its.temerity.TemerityLibraryKoinContext
import edu.ucsc.its.temerity.platformModule
import edu.ucsc.its.temerity.remote.PlatformApi
import edu.ucsc.its.temerity.remote.PlatformClientImpl
import edu.ucsc.its.temerity.remote.createPlatformApi
import io.ktor.client.HttpClient
import io.ktor.client.engine.HttpClientEngine
import io.ktor.client.plugins.HttpTimeout
import io.ktor.client.plugins.defaultRequest
import io.ktor.client.plugins.logging.LogLevel
import io.ktor.client.plugins.logging.Logger
import io.ktor.client.plugins.logging.Logging
import io.ktor.client.request.header
import org.koin.core.Koin
import org.koin.core.component.KoinComponent
import org.koin.core.parameter.parametersOf
import org.koin.dsl.module
import co.touchlab.kermit.Logger as KermitLogger

/**
 * This function provides the Koin module for the Temerity library.
 * It includes the platform module and provides factories for [HttpClient], [PlatformApi], [PlatformClient.Builder] and [PlatformClient].
 * Factories for the last two default to using [PlatformClientImpl] as the implementation.
 */
internal val libModule = module {
  includes(platformModule())
  factory { (enableNetworkLogs: Boolean, authToken: String) ->
    createHttpClient(get(), enableNetworkLogs, authToken)
  }
  factory { (serviceEndpoint: String, enableNetworkLogs: Boolean, authToken: String) ->
    val client: HttpClient = get { parametersOf(enableNetworkLogs, authToken) }
    val ktorfit = ktorfit {
      baseUrl(serviceEndpoint)
      httpClient(client)
      converterFactories(ApiResponseConverterFactory.create())
    }
    ktorfit.createPlatformApi()
  }
  factory<PlatformClient.Builder> {
    PlatformClientImpl.Builder()
  }
  factory<PlatformClient> { (serviceEndpoint: String, authToken: String) ->
    PlatformClientImpl.Builder().apply {
      serviceEndpoint(serviceEndpoint)
      serviceToken(authToken)
    }.build()
  }
}

/**
 * Adapted from https://github.com/joreilly/FantasyPremierLeague/
 * This function creates an HttpClient with the provided HttpClientEngine.
 * The HttpClientEngine is injected by the Koin platformModule(), which as its name suggests provides an HttpClientEngine implementation that is platform-dependent.
 * It configures the client to log all requests and responses if enableNetworkLogs is true. TODO: read this as a build setting
 * It also sets a default request header with the provided authToken.
 * @param httpClientEngine The HttpClientEngine to use for the HttpClient.
 * @param enableNetworkLogs Whether to log all requests and responses.
 * @param authToken The authToken to set as a default request header.
 * @return The created HttpClient.
 */
private fun createHttpClient(httpClientEngine: HttpClientEngine, enableNetworkLogs: Boolean, authToken: String) = HttpClient(httpClientEngine) {
  // Can't use install(ContentNegotiation){ json() } here because YuJa's API returns a 406 error if the Accept header is set to application/json
  // This requires JSON transformations to be done via manually-called library functions
  if (enableNetworkLogs) {
    install(Logging) {
      val kermit = KermitLogger(
        config = loggerConfigInit(
          platformLogWriter(NoTagFormatter),
          minSeverity = Severity.Info,
        ),
        tag = "TemerityLib",
      )
      logger = object : Logger {
        override fun log(message: String) {
          kermit.d(message)
        }
      }
      level = LogLevel.ALL
      sanitizeHeader { headerKey ->
        headerKey == "authToken"
      }
    }
  }

  install(HttpTimeout) {
    connectTimeoutMillis = TemerityLibraryKoinContext.TIMEOUT
    requestTimeoutMillis = TemerityLibraryKoinContext.TIMEOUT
    socketTimeoutMillis = TemerityLibraryKoinContext.TIMEOUT
  }
  // Pass the stored authToken as a header in every request
  defaultRequest {
    header("authToken", authToken)
  }
}

internal abstract class TemerityKoinComponent : KoinComponent {
  override fun getKoin(): Koin = TemerityLibraryKoinContext.koin
}
