/*
 *     Designed and developed in 2022-2024 by William Walker (wnwalker@ucsc.edu)
 *     Copyright 2022-2024 The Regents of the University of California. All rights reserved.
 *
 *     This library is free software; you can redistribute it and/or
 *     modify it under the terms of the GNU Lesser General Public
 *     License as published by the Free Software Foundation; version 2.1 of the License.
 *
 *     This library is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *     Lesser General Public License for more details.
 *
 *     You should have received a copy of the GNU Lesser General Public
 *     License along with this library; if not, write to the Free Software
 *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
package edu.ucsc.its.temerity

import co.touchlab.kermit.Logger
import com.skydoves.sandwich.StatusCode
import com.skydoves.sandwich.ktor.getStatusCode
import edu.ucsc.its.temerity.remote.NewUser
import io.kotest.core.spec.style.FunSpec
import io.kotest.matchers.file.shouldNotBeEmpty
import io.kotest.matchers.shouldBe
import kotlinx.coroutines.runBlocking
import kotlinx.datetime.Clock
import kotlinx.datetime.DateTimeUnit
import kotlinx.datetime.TimeZone
import kotlinx.datetime.minus
import kotlinx.datetime.plus
import kotlinx.datetime.toLocalDateTime
import kotlinx.datetime.todayIn
import org.dotenv.vault.dotenvVault
import org.jetbrains.kotlinx.dataframe.api.toDataFrame
import org.jetbrains.kotlinx.dataframe.io.writeCSV
import java.io.File
import java.util.UUID

class TemerityDevTest : FunSpec({

  val dotenv = dotenvVault()

  lateinit var classUnderTest: PlatformClient

  beforeTest {
    classUnderTest = platformClient {
      debugEnabled(true)
      serviceEndpoint(dotenv["YUJADEV_API_URL"])
      serviceToken(dotenv["YUJADEV_TOKEN"])
    }
  }

  test("Get all audit log entries of type \"New Login\" from the past month") {
    runBlocking {
      val currentDate = Clock.System.todayIn(TimeZone.currentSystemDefault())
      val pastDate = currentDate.minus(1, DateTimeUnit.MONTH)

      val returnedAuditLogEntries = classUnderTest.getAuditLogEntries(
        startTime = pastDate,
        endTime = currentDate,
        eventType = EventType.NEW_LOG_IN,
        sortOrder = AuditLogSortOrder.NEW_FIRST,
      )
      println("Retrieved audit log entries of type ${EventType.NEW_LOG_IN} from the past month:")
      returnedAuditLogEntries.forEach { println(it) }
      assert(returnedAuditLogEntries.isNotEmpty())
    }
  }
  test("Get all audit logs from the past month") {
    runBlocking {
      val currentDate = Clock.System.todayIn(TimeZone.currentSystemDefault())
      val pastDate = currentDate.minus(1, DateTimeUnit.MONTH)
      val logs = classUnderTest.getAuditLogEntries(pastDate, currentDate, sortOrder = AuditLogSortOrder.NEW_FIRST)
      println("Returned events from the past month:")
      logs.forEach { println(it) }
      assert(logs.isNotEmpty())
    }
  }
  test("Get all audit logs from the past day") {
    runBlocking {
      val currentDate = Clock.System.todayIn(TimeZone.currentSystemDefault())
      val pastDate = currentDate.minus(1, DateTimeUnit.DAY)
      val logs = classUnderTest.getAuditLogEntries(pastDate, currentDate, sortOrder = AuditLogSortOrder.NEW_FIRST)
      Logger.d("Returned events from the past day:")
      logs.forEach { Logger.d(it.toString()) }
      assert(logs.isNotEmpty())
    }
  }
  test("Get all recording sessions belonging to a single user scheduled for the next week") {
    runBlocking {
      val pastDate = Clock.System.todayIn(TimeZone.currentSystemDefault()).minus(1, DateTimeUnit.DAY)
      val futureDate = pastDate.plus(1, DateTimeUnit.MONTH)
      val sessions = classUnderTest.getUserSessions("dotenv[YUJADEV_TEST_USER]", pastDate, futureDate)
      Logger.d("Returned sessions for the next week:")
      sessions.forEach { Logger.d(it.toString()) }
      assert(sessions.isNotEmpty())
    }
  }

  test("PlatformClient can create a new user with not all JSON fields set") {
    runBlocking {
      val newUserRef = NewUser(
        emailAddress = "wnwalker+3@ucsc.edu",
        firstName = "Billy",
        lastName = "Jones",
        loginId = "teststudent123244",
        loginPassword = "asdadfadf",
        userType = "Student",
      )
      val response = classUnderTest.createUser(newUser = newUserRef)
      Logger.d("response: $response")
      response.getStatusCode().shouldBe(StatusCode.OK)
    }
  }

  test("Get all recording sessions on a single device scheduled for the next week") {
    runBlocking {
      val pastDate = Clock.System.todayIn(TimeZone.currentSystemDefault()).minus(1, DateTimeUnit.DAY)
      val futureDate = pastDate.plus(1, DateTimeUnit.MONTH)
      val testDeviceId = dotenv["YUJADEV_TEST_DEVICE_ID"].toLongOrNull()
      val sessions = classUnderTest.getDeviceSchedule(testDeviceId!!, pastDate, futureDate)
      val deviceName = classUnderTest.getDevice(testDeviceId).stationName
      println("Returned sessions for the device code 6489 with name $deviceName:")
      sessions.forEach { println(it) }
      assert(sessions.isNotEmpty())
    }
  }

  test("test datetime tostring formatting") {
    runBlocking {
      val time = Clock.System.now().toLocalDateTime(TimeZone.currentSystemDefault())
      println("Current time: $time")
    }
  }

  test("Get all audit log entries of type \"New Login\" from the past day") {
    runBlocking {
      val testEventType = EventType.NEW_LOG_IN
      val currentDate = Clock.System.todayIn(TimeZone.currentSystemDefault())
      val pastDate = currentDate.minus(1, DateTimeUnit.DAY)

      val returnedAuditLogEntries = classUnderTest.getAuditLogEntries(
        startTime = pastDate,
        endTime = currentDate,
        eventType = testEventType,
        sortOrder = AuditLogSortOrder.NEW_FIRST,
      )
      println("Retrieved audit log entries of type ${testEventType.value} from the past day:")
      returnedAuditLogEntries.forEach { println(it) }
      val df = returnedAuditLogEntries.toDataFrame()

      df.writeCSV("${testEventType.name}-Event-Report-${UUID.randomUUID()}-$pastDate-$currentDate-generated-${currentDate()}.csv")
      assert(returnedAuditLogEntries.isNotEmpty())
    }
  }

  test("PlatformClient can get the analytics storage report for a specific group") {
    val testGroup = dotenv["YUJADEV_TEST_GROUP"].toLong()
    runBlocking {
      val reportBytes = classUnderTest.getStorageAnalyticsReport(testGroup)
      val outputFile = File("storage-report-group-$testGroup-${currentDate()}-${UUID.randomUUID()}.csv")
      outputFile.outputStream().use { fileOut ->
        fileOut.write(reportBytes)
        fileOut.flush()
      }
      outputFile.shouldNotBeEmpty()
    }
  }

  test("PlatformClient can create new user") {
    runBlocking {
      val newUser = NewUser(
        emailAddress = "wnwalker+test1@ucsc.edu",
        firstName = "Test",
        lastName = "User",
        loginId = "testuser",
        loginPassword = "password",
        userType = "Student",
      )
      val response = classUnderTest.createUser(newUser = newUser)
      Logger.d(response.toString())
    }
  }
})
